/*
Copyright (c) 2001 Sebastian Zagrodzki

pty-redir98 is a rewritten version of old pty-redir program, using
Unix98 PTYs (/dev/ptmx) instead of traditional /dev/ttyp*

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>

extern int getpt(void);
extern char *ptsname(int);
extern int grantpt(int);
extern int unlockpt(int);

void assert(int val, char *msg)
{
	if (!val) {
		perror(msg);
		exit(1);
	}
}

int main(int argc,char **argv)
{
	int fd;
	char *name;

	if (argc < 2) {
		fprintf(stderr, "Not enough arguments\n");
		fprintf(stderr, "Usage: %s <command to execute>\n", *argv);
		exit(1);
	}
	assert((fd = getpt()) >= 0, "getpt");
	assert(grantpt(fd) == 0, "grantpt");
	assert(unlockpt(fd) == 0, "unlockpt");
    	assert((name = ptsname(fd)) != NULL, "ptsname");
	
	fprintf(stdout, "%s\n", name);
	fflush(stdout);
	
	assert(dup2(fd, 0) >= 0, "dup2(fd, 0)");
	assert(dup2(fd, 1) >= 0, "dup2(fd, 1)");
	assert(fflush(NULL) == 0, "fflush");

	switch (fork()) {
	case -1:
		perror("fork");
		exit(1);
		break;
	case 0:
		argv++;
		assert(execvp(*argv,argv) == 0, "execvp");
		break;
	default:
		break;
	}
	return 0;
}
